"""pyfan generate and clean pdf files from folder

Gather all tex files from a folder, allow for exclusion strings. Generate PDFs
from the tex files. And then clean up extraneous PDF outputs.
"""

import subprocess
import os
from pathlib import Path


def ff_pdf_gen_clean(ls_spt_srh=None,
                     spt_out='C:/Users/fan/Documents/Dropbox (UH-ECON)/' +
                             'Project Emily Minority Survey/' +
                             'EthLang/reg_lang_abi_cls/',
                     spn_pdf_exe='C:/texlive/2019/bin/win32/xelatex.exe',
                     ls_st_contain=None,
                     ls_st_ignore=None,
                     bl_recursive=False,
                     bl_clean=True,
                     ls_suf_clean=None):
    """Generate pdf files from latex files in various folders.

    Parameters
    ----------
    ls_spt_srh : :obj:`list` of :obj:`str`
        A list of strings of the path in which to search for tex files.
    spt_out : str
        The Path to store outputs. All PDFs stored under single directory.
    spn_pdf_exe: str
        The path to the pdflatex or alternative exe file
    ls_st_contain: :obj:`list` of :obj:`str`
        a list of strings the found names must contain one of these search words
    ls_st_ignore : :obj:`list` of :obj:`str`
        a list of string file names to ignore
    bl_recursive : bool
        Whether to search for all tex files within subfolders
    bl_clean : bool
        To clean up after file generation
    ls_suf_clean: :obj:`list` of :obj:`str`
        list of
    """

    # Set mutables
    if ls_spt_srh is None:
        ls_spt_srh = ['C:/Users/fan/Documents/Dropbox (UH-ECON)/' +
                      'Project Emily Minority Survey/' +
                      'EthLang/reg_lang_abi_cls/']
    if ls_st_contain is None:
        ls_st_contain = None
        ls_st_contain = ['t6']
    if ls_st_ignore is None:
        ls_st_ignore = ['all.tex']
    if ls_suf_clean is None:
        ls_suf_clean = ['aux', 'log', 'out', 'bbl', 'blg']

    # Search for tex files in folders
    if bl_recursive:
        # use rglob
        ls_spn_found = [spn_file
                        for spt_srh in ls_spt_srh
                        for spn_file in Path(spt_srh).rglob('*.tex')]
    else:
        # use glob
        ls_spn_found = [spn_file
                        for spt_srh in ls_spt_srh
                        for spn_file in Path(spt_srh).glob('*.tex')]

    # Generate pdf from found tex files
    for spn_found in ls_spn_found:
        # convert WindosPath to String
        spn_found = str(spn_found)

        # search through ignore list
        if sum([st_ignore in spn_found
                for st_ignore in ls_st_ignore]):
            pass
        else:

            if (ls_st_contain is None) or \
                    (sum([st_contain in spn_found
                          for st_contain in ls_st_contain]) > 0):

                # change to local directory
                os.chdir(os.path.split(spn_found)[0])

                # generate pdf
                subprocess.call([spn_pdf_exe, '-output-directory',
                                 spt_out, spn_found], shell=False)

                # path no suffix
                if bl_clean:
                    path_no_suffix = os.path.splitext(spn_found)[0]
                    ls_st_remove_suffix = ls_suf_clean
                    for st_suffix in ls_st_remove_suffix:
                        srn_cur_file = path_no_suffix + "." + st_suffix
                        if os.path.isfile(srn_cur_file):
                            os.remove(path_no_suffix + "." + st_suffix)


if __name__ == '__main__':
    fp_pdf_gen_clean()



# Generated with SMOP  0.41-beta
from libsmop import *
# solver.m

    
@function
def solver(ai=None,af=None,w=None,*args,**kwargs):
    varargin = solver.varargin
    nargin = solver.nargin

    rand(1,2,3)
    
    # Copyright 2004 The MathWorks, Inc.
    
    nBlocks=max(ravel(ai))
# solver.m:6
    m,n=size(ai,nargout=2)
# solver.m:7
    # Make increment tables
# N=1, E=2, S=3, W=4
    I=concat([0,1,0,- 1])
# solver.m:11
    J=concat([1,0,- 1,0])
# solver.m:12
    a=copy(ai)
# solver.m:14
    mv=[]
# solver.m:15
    while logical_not(isequal(af,a)):

        # Pick a random block
        bid=ceil(dot(rand(),nBlocks))
# solver.m:19
        i,j=find(a == bid,nargout=2)
# solver.m:20
        r=ceil(dot(rand(),4))
# solver.m:23
        ni=i + I(r)
# solver.m:24
        nj=j + J(r)
# solver.m:25
        if (ni < 1) or (ni > m) or (nj < 1) or (nj > n):
            continue
        # Is it a legal move? Check for collision
        if a(ni,nj) > 0:
            continue
        # Check distance
    # Get the target location
        ti,tj=find(af == bid,nargout=2)
# solver.m:38
        d=(ti - i) ** 2 + (tj - j) ** 2
# solver.m:39
        dn=(ti - ni) ** 2 + (tj - nj) ** 2
# solver.m:40
        if (d < dn) and (rand() > 0.05):
            continue
        # Move the block
        a[ni,nj]=bid
# solver.m:47
        a[i,j]=0
# solver.m:48
        mv[end() + 1,concat([1,2])]=concat([bid,r])
# solver.m:51

    
    return mv
    
if __name__ == '__main__':
    pass
    
    ## function r = rand(varargin)
##     global s1 s2 s3
##     if nargin != 0
##         r=0;
##         s1=varargin{1};
##         s2=varargin{2};
##         s3=varargin{3};
##     else
##         [r,s1,s2,s3] = r8_random(s1,s2,s3);
##     end
## end


# cd "C:/Users/fan/Documents/Dropbox (UH-ECON)/repos/Math4Econ/calconevar"
# pandoc -s interval.tex -o interval.md
# pandoc -s localglobal.tex -o localglobal.md
# pandoc -s polynomial.tex -o polynomial.md
#
# cd "C:/Users/fan/Documents/Dropbox (UH-ECON)/repos/Math4Econ/calconevar"
# pandoc -s polynomial.tex -o polynomial.md
#
# replace matlabcode by lstlisting
# replace matlabtitle by subsubsection
# replace matlabtitle by subsubsection
# replace matlabheading by paragraph
# replace [width=/maxwidth{56.196688409433015em}]{
#     by {polynomial_image/
#
# 1. convert mlx to tex
# 2. tex replace as above
# 3. convert to md
# 4. md add front matter as separate files, to be merged with md output here.

# spt_convert_tex = 'C:/Users/fan/Documents/Dropbox (UH-ECON)/repos/Math4Econ/calconevar/'
# snm_mat_tex_auto = 'interval.tex'
# spn_convert_tex = spt_convert_tex + snm_mat_tex_auto
#
# with open(spn_convert_tex) as fl_tex:
#     fl_tex_new = fl_tex.read().replace('matlabcode', 'lstlisting')
#
# with open(spn_convert_tex, "w") as f:
#     f.write(fl_tex_new)
#
# from pathlib import Path

from pathlib import Path
import subprocess
import os


def ff_mlxtex2md():
    verbose = False

    # Directories to search in
    spt_root = "C:/Users/fan/Documents/Dropbox (UH-ECON)/repos/Math4Econ/"
    ls_srt_subfolders = ['calconevar/', 'derivative/', 'derivative_application/',
                         'matrix_basics/',
                         'opti_firm_constrained/', 'opti_hh_constrained_brsv/',
                         'opti_hh_constrained_brsv_inequality/']
    ls_spt = [spt_root + srt_subf for srt_subf in ls_srt_subfolders]
    print(ls_spt)

    # get file names
    ls_spn_found_tex = [spn_file for spt_srh in ls_spt
                        for spn_file in Path(spt_srh).rglob('*.tex')]
    print(ls_spn_found_tex)

    for spn_found_tex in ls_spn_found_tex:
        if verbose:
            print(spn_found_tex)
        fl_tex_contents = open(spn_found_tex)
        stf_tex_contents = fl_tex_contents.read()

        # file image folder
        sna_file = Path(spn_found_tex).stem

        # define new and old
        ls_st_old = ['matlabcode', 'matlabtitle', 'matlabheading',
                     '[width=\\maxwidth{56.196688409433015em}]{']
        ls_st_new = ['lstlisting', 'subsubsection', 'paragraph',
                     'by {' + sna_file + '_image/']

        # zip and loop and replace
        for old, new in zip(ls_st_old, ls_st_new):
            stf_tex_contents = stf_tex_contents.replace(old, new)
        if verbose:
            print(stf_tex_contents)

        # convert from tex to md
        path_no_suffix = os.path.splitext(spn_found_tex)[0]
        spn_md = path_no_suffix + '.md'
        print(spn_found_tex)
        print(spn_md)
        subprocess.call(['pandoc', '-s', str(spn_found_tex), '-o', str(spn_md)])

    # m4econ to root
    # with several testing md files with preamble/frontmatter, generate bookdown
    # after generating bookdown, add in personalized preamble.
    # generate a folder with only preambles, and merge the md files created here with those


if __name__ == '__main__':
    ff_mlxtex2md()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyfan',
 'pyfan.pkgtest.smop',
 'pyfan.util',
 'pyfan.util.pdf',
 'pyfan.util.rmd']

package_data = \
{'': ['*']}

install_requires = \
['python-frontmatter>=0.5.0,<0.6.0', 'pyyaml>=5.3.1,<6.0.0']

setup_kwargs = {
    'name': 'pyfan',
    'version': '0.1.33',
    'description': '',
    'long_description': None,
    'author': 'Fan Wang',
    'author_email': 'wangfanbsg75@live.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
